#include <QVBoxLayout>
#include <QDebug>
#include <QMouseEvent>

#include "video.h"
#include "settings.h"


/*************
 * VideoImpl *
 *************/

class VideoImpl
{
public:

    VideoImpl(PlaylistItem* item, Video* parent)
        : m_parent(parent)
        , m_uri(item->uri()) {}

    virtual ~VideoImpl(void) {}

    Video* parent(void) const
    { return m_parent; }

    QString const& uri(void) const
    { return m_uri; }

    void end(void)
    { emit parent()->end(); }

    virtual void pause(void) = 0;
    virtual void resume(void) = 0;
    virtual bool onTimeout(void) = 0;
    virtual bool start(void) = 0;
    virtual bool stop(void) = 0;

private:

    Video* m_parent;
    QString m_uri;
};

/****************
 * VLCVideoImpl *
 ****************/

class VLCVideoImpl
    : public VideoImpl
{
public:

    VLCVideoImpl(PlaylistItem* item, Video* p)
        : VideoImpl(item, p)
    {
        static char const* const vlc_args[] = {
            "-I", "dummy",
            "--no-xlib",
            "--ignore-config",
            "--no-video-title-show",
            "--no-disable-screensaver",
        #ifdef __arm__
            "--vout", "omxil_vout",
        #endif
        };

        parent()->setAutoFillBackground(true);
        QPalette plt;
        plt.setColor(QPalette::Window, Qt::black);
        parent()->setPalette(plt);

        m_instance = libvlc_new(
            sizeof(vlc_args)/sizeof(*vlc_args),
            vlc_args);

        m_player = libvlc_media_player_new(m_instance);

        // Set callback on end of media.
        {
            libvlc_event_manager_t *em =
                libvlc_media_player_event_manager(m_player);

            libvlc_event_attach(
                em, libvlc_MediaPlayerEndReached,
                VLCVideoImpl::onMediaEndReached,
                this);
        }
    }

    ~VLCVideoImpl(void)
    {
        libvlc_media_player_stop(m_player);
        libvlc_media_player_release(m_player);
        libvlc_release(m_instance);
    }

    static void onMediaEndReached(libvlc_event_t const* e, void* self)
    { reinterpret_cast<VLCVideoImpl*>(self)->onMediaEndReached(); }

    void onMediaEndReached(void) { end(); }

    bool start(void)
    {
        m_media = libvlc_media_new_location(m_instance, uri().toUtf8().data());
        libvlc_media_player_set_media(m_player, m_media);
        #ifndef __arm__
        libvlc_media_player_set_xwindow(m_player, parent()->winId());
        #endif
        libvlc_media_player_play(m_player);

        return true;
    }

    bool stop(void)
    {
        libvlc_media_player_stop(m_player);
        libvlc_media_release(m_media);
        return true;
    }

    bool onTimeout(void)
    { end(); return true; }

    void pause(void)
    { libvlc_media_player_pause(m_player); }

    void resume(void)
    { libvlc_media_player_pause(m_player); }

private:

    libvlc_instance_t*     m_instance;
    libvlc_media_player_t* m_player;
    libvlc_media_t*        m_media;
};


/*********
 * Video *
 *********/


Video::Video(PlaylistItem *item, QWidget *parent)
    : QWidget(parent)
    , Runnable(item->time())
{
    m_impl = new VLCVideoImpl(item, this);
}


Video::~Video(void)
{ delete m_impl; }


bool Video::startImpl(void)
{ return m_impl->start(); }


bool Video::stopImpl(void)
{ return m_impl->stop(); }


bool Video::onTimeoutImpl(void)
{ return m_impl->onTimeout(); }


void Video::pauseImpl(void)
{ m_impl->pause(); }


void Video::resumeImpl(void)
{ m_impl->resume(); }


void Video::mousePressEvent(QMouseEvent* event)
{
    if (event->button() == Qt::RightButton)
        emit rightclick();
}